package de.intevation.flys.artifacts.model.fixings;

import org.apache.log4j.Logger;

import de.intevation.artifacts.Artifact;
import de.intevation.artifacts.CallContext;
import de.intevation.flys.artifacts.FLYSArtifact;
import de.intevation.flys.artifacts.model.CalculationResult;
import de.intevation.flys.artifacts.model.DataFacet;
import de.intevation.flys.artifacts.model.FacetTypes;
import de.intevation.flys.artifacts.states.DefaultState.ComputeType;
import de.intevation.flys.utils.KMIndex;


/**
 * Facet to show W values for Q values at km for a date.
 *
 * @author <a href="mailto:raimund.renkert@intevation.de">Raimund Renkert</a>
 */
public class FixReferenceEventsFacet
extends      DataFacet
implements   FacetTypes {

    /** House logger. */
    private static Logger logger = Logger.getLogger(FixReferenceEventsFacet.class);

    /** Trivial Constructor. */
    public FixReferenceEventsFacet() {
    }


    /**
     * @param name
     */
    public FixReferenceEventsFacet(int index, String name, String description) {
        super(index,
             name,
             description,
             ComputeType.ADVANCE,
             null,
             null);
    }


    /**
     * Returns the data this facet requires.
     *
     * @param artifact the owner artifact.
     * @param context  the CallContext (ignored).
     *
     * @return the data.
     */
    @Override
    public Object getData(Artifact artifact, CallContext context) {
        logger.debug("FixReferenceEventsFacet.getData");

        if (artifact instanceof FLYSArtifact) {
            FLYSArtifact flys = (FLYSArtifact)artifact;

            CalculationResult res =
                (CalculationResult) flys.compute(context,
                                                 ComputeType.ADVANCE,
                                                 false);

            FixResult result = (FixResult) res.getData();
            double currentKm =
                ((Double)context.getContextValue("currentKm")).doubleValue();

            KMIndex<QWD []> kmQWs = result.getReferenced();
            KMIndex.Entry<QWD []> kmQWsEntry = kmQWs.binarySearch(currentKm);
            QWD[] qwds = null;
            if (kmQWsEntry != null) {
                int ndx = index & 255;
                qwds = kmQWsEntry.getValue();
                for (int i = 0; i < qwds.length; i++) {
                    if (qwds[i].getIndex() == ndx) {
                        return qwds[i];
                    }
                }
                return null;
            }
            return null;
        }
        else {
            logger.debug("Not an instance of FixationArtifact.");
            return null;
        }
    }


    /**
     * Create a deep copy of this Facet.
     * @return a deep copy.
     */
    @Override
    public FixReferenceEventsFacet deepCopy() {
        FixReferenceEventsFacet copy = new FixReferenceEventsFacet();
        copy.set(this);
        return copy;
    }
}
// vim:set ts=4 sw=4 si et sta sts=4 fenc=utf8 :
